import { spawnSync } from 'child_process';
import { unlinkSync, readFileSync } from 'fs';

export enum VideoType {
  YOUTUBE = 'youtube',
}

interface ClipVideoBaseParams {
  url: string,
  start: number | string,
  end: number | string,
}

interface ClipYouTubeVideoParams extends ClipVideoBaseParams {
  downloaderExecutablePath?: string;
}

interface ClipVideoParams extends ClipYouTubeVideoParams {
  type: VideoType,
}

const clipYouTubeVideo = (params: ClipYouTubeVideoParams) => {
  if (!params.downloaderExecutablePath) {
    throw new Error('Downloader not found.');
  }
  spawnSync(
    params.downloaderExecutablePath,
    [
      '--postprocessor-args',
      `-ss ${params.start.toString()} -to ${params.end.toString()}`,
      '-o',
      'output.webm',
      params.url,
    ],
  );
  const output = readFileSync('output.webm');
  unlinkSync('output.webm');
  return output;
};

export const clipVideo = (params: ClipVideoParams) => {
  const {
    type: videoType, url, start, end,
    downloaderExecutablePath,
  } = params;

  switch (videoType as string) {
    case VideoType.YOUTUBE:
      return clipYouTubeVideo({
        downloaderExecutablePath,
        url,
        start,
        end,
      });
    default:
      break;
  }

  throw new TypeError(`Invalid video type: "${videoType}". Valid values are: ${JSON.stringify(Object.values(VideoType))}`);
};
