import {
  describe, it, expect, vi, beforeEach, Mock,
} from 'vitest';
import { spawnSync } from 'child_process';
import { readFileSync, unlinkSync } from 'fs';
import * as youtubeClipCore from '../src';

vi.mock('child_process', () => ({
  spawnSync: vi.fn(),
}));

vi.mock('fs', () => ({
  readFileSync: vi.fn(),
  unlinkSync: vi.fn(),
}));

describe('clipVideo', () => {
  beforeEach(() => {
    (readFileSync as Mock).mockReturnValueOnce(Buffer.from(''));
  });

  it('calls the downloader function', () => {
    youtubeClipCore.clipVideo({
      downloaderExecutablePath: 'yt-dlp',
      type: youtubeClipCore.VideoType.YOUTUBE,
      url: 'https://www.youtube.com/watch?v=BaW_jenozKc',
      start: 0,
      end: 0,
    });

    expect(spawnSync).toBeCalledWith('yt-dlp', expect.anything());
  });

  it('calls the buffer extract function', () => {
    youtubeClipCore.clipVideo({
      downloaderExecutablePath: 'yt-dlp',
      type: youtubeClipCore.VideoType.YOUTUBE,
      url: 'https://www.youtube.com/watch?v=BaW_jenozKc',
      start: 0,
      end: 0,
    });

    expect(readFileSync).toBeCalled();
  });

  it('calls the cleanup function', () => {
    youtubeClipCore.clipVideo({
      downloaderExecutablePath: 'yt-dlp',
      type: youtubeClipCore.VideoType.YOUTUBE,
      url: 'https://www.youtube.com/watch?v=BaW_jenozKc',
      start: 0,
      end: 0,
    });

    expect(unlinkSync).toBeCalled();
  });
});
